package jsetl.lib;

import jsetl.*;
import jsetl.exception.NotDefConstraintException;

/* 
 * User-defined constraints implementing operations
 * on integer numbers.
 * 
 */

public class MathOps extends NewConstraints {
    public MathOps(jsetl.Solver slv)
    {
        super(slv);
    }
  
    public Constraint absTest(IntLVar x, IntLVar y) { 
        return new Constraint("absTest", x, y); 
    }

    public Constraint abs(IntLVar x, IntLVar y) { 
        return new Constraint("absND", x, y); 
    }

    protected void user_code(Constraint constraint) throws NotDefConstraintException
    {
        if (constraint.getName() == "absTest") absTest(constraint);
        else if (constraint.getName() == "absND") abs(constraint);
        else throw new NotDefConstraintException();
    }
    
    private void absTest(Constraint c)  // x = |y|
     {
        IntLVar x = (IntLVar)c.getArg(1); 
        IntLVar y = (IntLVar)c.getArg(2); 
        if (!x.isBound()) {            // irreducible case
            c.notSolved(); 
            return;
        };  
        if (x.getValue() >= 0) 
            solver.add(x.eq(y));
        else
            solver.add(x.eq(new IntLVar(0).sub(y)));
        return;
    }
    
    private void abs(Constraint c)  // x = |y|
     {  
        IntLVar x = (IntLVar)c.getArg(1); 
        IntLVar y = (IntLVar)c.getArg(2); 
        switch (c.getAlternative()) {
        case 0:
            solver.addChoicePoint(c);
            solver.add(x.ge(0).and(x.eq(y)));
            break;
        case 1:
            solver.add(x.lt(0).and(x.eq(new IntLVar(0).sub(y))));
        }
        return;
    }

}
